<?php

namespace App\Http\Controllers\V1\Seguridad;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Password;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;

class ResetPasswordController extends Controller
{
    public function reset(Request $request)
    {
        $data = $request->only('email', 'password', 'password_confirmation', 'token');
        //Realizamos las validaciones
        $validator = Validator::make($data, [
            'token' => 'required',
            'email' => 'required|email|exists:users,email',
            'password' => 'required|confirmed|min:6',
        ]);
        //Devolvemos un error si fallan las validaciones
        if ($validator->fails()) {
            return response()->json($validator->messages(), 400);
        }
        $response = $this->broker()->reset(
            $data,
            function ($user, $password) {
                $user->password = bcrypt($password);
                $user->save();
            }
        );

        return $response == Password::PASSWORD_RESET
            ? response()->json([
                'message' => 'Se ha restablecido la contraseña correctamente'
            ], Response::HTTP_OK)
            : response()->json('No se pudo restablecer la contraseña', 401);
    }

    public function broker()
    {
        return Password::broker();
    }
}
