<?php

namespace App\Repositories;

use App\Dto\InsigniaDto;
use App\Interfaces\InsigniaInterface;
use App\Models\Insignia;

class InsigniaRepository implements InsigniaInterface
{
    public function allInsignia()
    {
        $insignias = Insignia::with(['files', 'recompensa'])->get();
        return $insignias;
    }

    public function detailInsignia(string $idInsignia)
    {
        $insigniaQuery = Insignia::with(['files', 'recompensa'])
            ->where('id', $idInsignia)->first();
        return $insigniaQuery;
    }

    public function createInsignia(InsigniaDto $insigniaDto)
    {
        $insigniaNew = Insignia::create([
            "nombre" => $insigniaDto->nombre,
            "objetivo" => $insigniaDto->objetivo,
            "user_id" => $insigniaDto->userId,
            "file_id" => $insigniaDto->fileId,
            "recompensa_id" => $insigniaDto->recompensaId,
        ]);
        return $insigniaNew;
    }

    public function updateInsignia(InsigniaDto $insigniaDto, int $idInsignia)
    {
        $insigniaFind = Insignia::findOrFail($idInsignia);
        $insigniaFind->update([
            "nombre" => $insigniaDto->nombre,
            "objetivo" => $insigniaDto->objetivo,
            "file_id" => $insigniaDto->fileId,
            "recompensa_id" => $insigniaDto->recompensaId,
            "estatus" => $insigniaDto->estatus
        ]);
        return $insigniaFind;
    }

    public function deleteInsignia(string $idInsignia)
    {
        $insigniaDelete = Insignia::findOrfail($idInsignia);
        $insigniaDelete->delete();
    }

    public function findInsigniasByPlayer(string $idPlayer){
        $insignias = Insignia::leftJoin('files', 'insignias.file_id', '=', 'files.id')
        ->whereHas('niveles_retos.recompensa', function ($query) use ($idPlayer) {
            $query->where('participante_id', $idPlayer);
        })
        ->select('insignias.nombre', 'insignias.objetivo', 'files.path')
        ->get();
        return $insignias;
    }
}
