<?php

namespace App\Repositories;

use App\Dto\NivelDto;
use App\Interfaces\NivelRetoInterface;
use App\Models\NivelReto;
use Carbon\Carbon;

class NivelRepository implements NivelRetoInterface
{
    public function filterNivelGetDeleteWhereNotIds(string $retoId, $idsNotDelete)
    {
        $nivelesQueryDelete = NivelReto::where('reto_id', $retoId)
            ->whereDoesntHave('recompensa')
            ->whereNotIn('id', $idsNotDelete)
            ->get();
        return $nivelesQueryDelete;
    }

    public function createNivel(NivelDto $nivelDto)
    {
        $newNivel = NivelReto::create([
            "nombre" => $nivelDto->nombre,
            "meta" => $nivelDto->meta,
            "puntos" => $nivelDto->puntos,
            "fecha_inicio" => $nivelDto->fechaInicio,
            "fecha_fin" => $nivelDto->fechaFin,
            "insignia_id" => $nivelDto->insigniaId,
            "reto_id" => $nivelDto->retoId,
        ]);
        return $newNivel;
    }

    public function findNivel(string $idNivel)
    {
        $nivel = NivelReto::findOrFail($idNivel);
        $nivelDto = new NivelDto();
        $nivelDto->id = $nivel->id;
        $nivelDto->nombre = $nivel->nombre;
        $nivelDto->meta = $nivel->meta;
        $nivelDto->puntos = $nivel->puntos;
        $nivelDto->fechaInicio = $nivel->fecha_inicio;
        $nivelDto->fechaFin = $nivel->fecha_fin;
        $nivelDto->insigniaId = $nivel->insignia_id;
        return $nivelDto;
    }

    public function updateNivel(nivelDto $nivelDto, string $idNivel)
    {
        $nivelUpdate = NivelReto::findOrFail($idNivel);
        $nivelUpdate->update([
            "nombre" => $nivelDto->nombre,
            "meta" => $nivelDto->meta,
            "puntos" => $nivelDto->puntos,
            "fecha_inicio" => $nivelDto->fechaInicio,
            "fecha_fin" => $nivelDto->fechaFin,
            "insignia_id" => $nivelDto->insigniaId
        ]);
        return $nivelUpdate;
    }

    public function deleteNivel(string $idNivel)
    {
        $nivelFind = NivelReto::findOrFail($idNivel);
        $nivelFind->delete();
    }

    public function totalPointsByRole(string $role)
    {
        $totalPuntos = NivelReto::whereHas("recompensa.participante.usuario.roles", function ($query) use ($role) {
            $query->where('name', $role);
        })->whereHas('recompensa')
        ->get()->pluck('puntos')->sum();
        return $totalPuntos;
    }

    public function totalPointsByPlayer(string $idPlayer)
    {
        $totalPuntos = NivelReto::whereHas('recompensa', function ($query) use ($idPlayer) {
            $query->where('historial_recompensas.participante_id', $idPlayer);
        })
            ->get()->pluck('puntos')->sum();
        return $totalPuntos;
    }

    public function totalPoinstWeekendByRole(string $role)
    {
        $currentDateTime = Carbon::now();
        $inicioSemana = $currentDateTime->copy()->startOfWeek();
        $finSemana = $currentDateTime->copy()->endOfWeek();
        $puntoSemana =  NivelReto::whereHas('recompensa', function ($query) use ($inicioSemana, $finSemana) {
            $query->whereBetween('historial_recompensas.created_at', [$inicioSemana, $finSemana]);
        })
        ->whereHas("recompensa.participante.usuario.roles", function ($query) use ($role) {
            $query->where('name', $role);
        })
        ->get()->pluck('puntos')->sum();
        return $puntoSemana;
    }

    public function totalPoinstWeekendByPlayer(string $idPlayer)
    {
        $currentDateTime = Carbon::now();
        $inicioSemana = $currentDateTime->copy()->startOfWeek();
        $finSemana = $currentDateTime->copy()->endOfWeek();
        $puntoSemana =  NivelReto::whereHas('recompensa', function ($query) use ($idPlayer, $inicioSemana, $finSemana) {
            $query->where('historial_recompensas.participante_id', $idPlayer)
            ->whereBetween('historial_recompensas.created_at', [$inicioSemana, $finSemana]);
        })
        ->get()->pluck('puntos')->sum();
        return $puntoSemana;
    }

    public function totalPoinstMonthByRole(string $role)
    {
        $currentDateTime = Carbon::now();
        $inicioMes = $currentDateTime->copy()->startOfMonth();
        $finMes = $currentDateTime->copy()->endOfMonth();
        $puntosMes = NivelReto::whereHas('recompensa', function ($query) use ($inicioMes, $finMes) {
            $query->whereBetween('historial_recompensas.created_at', [$inicioMes, $finMes]);
        })
        ->whereHas("recompensa.participante.usuario.roles", function ($query) use ($role) {
            $query->where('name', $role);
        })
        ->get()->pluck('puntos')->sum();
        return $puntosMes;
    }

    public function totalPoinstMonthByPlayer(string $idPlayer)
    {
        $currentDateTime = Carbon::now();
        $inicioMes = $currentDateTime->copy()->startOfMonth();
        $finMes = $currentDateTime->copy()->endOfMonth();
        $puntosMes = NivelReto::whereHas('recompensa', function ($query) use ($idPlayer, $inicioMes, $finMes) {
            $query->where('historial_recompensas.participante_id', $idPlayer)
            ->whereBetween('historial_recompensas.created_at', [$inicioMes, $finMes]);
        })
            ->get()->pluck('puntos')->sum();
        return $puntosMes;
    }

    public function findNivelsWhereNotRecompensaByEvidenciaId(string $fechaCapturaEvidencia, string $idEvidencia)
    {
        $queryNiveles = NivelReto::with(['reto_detail'])
            ->whereHas('reto_detail.evidencias_retos', function ($query) use ($idEvidencia) {
                $query->where('evidencia_id', $idEvidencia);
            })
            ->whereDoesntHave('recompensa')
            ->where('nivel_retos.fecha_inicio', '<=', $fechaCapturaEvidencia)
            ->where('nivel_retos.fecha_fin', '>=', $fechaCapturaEvidencia)
            ->get();
        return $queryNiveles;
    }
}
