<?php

namespace App\Repositories;

use App\Dto\PlayerDto;
use App\Interfaces\PlayerInterface;
use App\Models\Participante;
use Illuminate\Support\Facades\DB;

class PlayerRepository implements PlayerInterface
{

    public function allPlayers()
    {
        $allParticipantes = Participante::with(['retos', 'files', 'usuario.roles'])->get();
        return $allParticipantes;
    }

    public function findPlayersByRole(string $role)
    {
        $queryPlayer = Participante::with(['retos', 'files', 'usuario.roles'])
            ->whereHas('usuario.roles', function ($query) use ($role) {
                $query->where('name', $role);
            })
            ->get();
        return $queryPlayer;
    }

    public function findPlayerById(string $playerId)
    {
        $queryPlayer = Participante::with(['retos', 'files', 'usuario.roles'])
            ->where('id', $playerId)->first();
        return $queryPlayer;
    }

    public function findPlayerWithUserOnly(string $playerId)
    {
        $queryPlayer = Participante::with("usuario")
        ->where("id", $playerId)->first();
        return $queryPlayer;
    }

    public function findPlayerByUserId(string $userId)
    {
        $queryPlayer = Participante::leftJoin('files', 'participantes.file_id', '=', 'files.id')
            ->select(
                'participantes.id',
                'participantes.nombre_tienda',
                'participantes.nombre_completo',
                'participantes.notificacion',
                'files.path'
            )
            ->where('participantes.user_id', $userId)
            ->first();
        return $queryPlayer;
    }

    public function createPlayer(PlayerDto $playerDto)
    {
        $participante = Participante::create([
            "nombre_tienda" => $playerDto->nombreTienda,
            "direccion" => $playerDto->direccion,
            "nombre_completo" => $playerDto->nombreCompleto,
            "localidad" => $playerDto->localidad,
            "estado" => $playerDto->estado,
            "ciudad" => $playerDto->ciudad,
            "ruta" => $playerDto->ruta,
            "file_id" => $playerDto->fileId,
            "user_id" => $playerDto->userId,
            "identificador" => $playerDto->identificador
        ]);
        return $participante;
    }

    public function updatePlayer(PlayerDto $playerDto, int $id)
    {
        $participanteFind = Participante::findOrFail($id);
        $participanteFind->update([
            "nombre_tienda" => $playerDto->nombreTienda,
            "direccion" => $playerDto->direccion,
            "nombre_completo" => $playerDto->nombreCompleto,
            "localidad" => $playerDto->localidad,
            "estado" => $playerDto->estado,
            "ciudad" => $playerDto->ciudad,
            "ruta" => $playerDto->ruta,
            "file_id" => $playerDto->fileId
        ]);
        return $participanteFind;
    }

    public function updatePlayerFindUserId(PlayerDto $playerDto, string $userId)
    {
        $participanteFind = Participante::where("user_id", $userId)->first();
        $participanteFind->update([
            "nombre_tienda" => $playerDto->nombreTienda,
            "direccion" => $playerDto->direccion,
            "nombre_completo" => $playerDto->nombreCompleto,
            "localidad" => $playerDto->localidad,
            "estado" => $playerDto->estado,
            "ciudad" => $playerDto->ciudad,
            "ruta" => $playerDto->ruta,
            "file_id" => $playerDto->fileId
        ]);
        return $participanteFind;
    }


    public function deletePlayer(int $idParticipante)
    {
        $participanteFind = Participante::findOrFail($idParticipante);
        $participanteFind->delete();
    }

    public function getRankingByTypePlayer(string $typeUser)
    {
        $listVentaTotalParticipante = Participante::whereHas('usuario.roles', function ($query) use ($typeUser) {
            $query->where('name', $typeUser);
        })
            ->join('evidencias', 'participantes.id', '=', 'evidencias.participante_id')
            ->join('evidencia_registros', 'evidencia_registros.evidencia_id', '=', 'evidencias.id')
            ->leftJoin('files', 'participantes.file_id', '=', 'files.id')
            ->where('evidencias.estatus', 'aprobado')
            ->select(
                'participantes.id',
                'participantes.nombre_tienda',
                'participantes.nombre_completo',
                'files.path',
                DB::raw('SUM(evidencia_registros.cantidad) as cantidad_total'),
                DB::raw('SUM(evidencia_registros.monto) as monto_total'),
                DB::raw('(SELECT SUM(nivel_retos.puntos) FROM nivel_retos 
                INNER JOIN historial_recompensas ON nivel_retos.id = historial_recompensas.nivel_id 
                WHERE historial_recompensas.participante_id = participantes.id) as puntos')
            )
            ->groupBy('participantes.id', 'participantes.nombre_tienda', 'participantes.nombre_completo')
            ->orderBy('cantidad_total', 'desc')
            ->orderBy('monto_total', 'desc')
            ->orderBy('puntos', 'desc')
            ->get();
        return $listVentaTotalParticipante;
    }
}
