<?php

namespace App\Repositories;

use App\Dto\UserActiveDto;
use App\Dto\UserDto;
use App\Interfaces\UserInterface;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class UserRepository implements UserInterface
{

    public function createUserRol(UserDto $userDto, string $rol = null)
    {
        $user = User::create([
            'email' => $userDto->email,
            'name' => $userDto->name,
            'password' => bcrypt($userDto->password)
        ]);
        if ($rol)
            $user->assignRole($rol);
        return $user;
    }

    public function updateUser(UserDto $userDto, string $idUser)
    {
        $dataUpdate = [
            "name" => $userDto->name
        ];
        if ($userDto->email !== "") {
            $dataUpdate["email"] = $userDto->email;
        }
        $userUpdate = User::findOrFail($idUser);
        $userUpdate->update($dataUpdate);
        return $userUpdate;
    }

    public function updateCountSession(string $idUser)
    {
        $userCurrent = User::findOrFail($idUser);
        $lastLogin = Carbon::parse($userCurrent->last_login);
        if (!$lastLogin->isToday()) {
            $userCurrent->update([
                'logins_count' =>  $userCurrent->logins_count + 1,
                'last_login' => Carbon::now()
            ]);
        }
    }

    public function listUsersWithActive()
    {
        $currentDateTime = Carbon::now();
        $inicioMes = $currentDateTime->copy()->startOfMonth();
        $finMes = $currentDateTime->copy()->endOfMonth();

        $usersQuery = User::whereHas('roles', function ($query) {
            $query->where('name', '!=', 'administrador');
        })
        ->with("participante.evidencias", function($query) use($inicioMes, $finMes) {
            $query->whereBetween('evidencias.created_at', [$inicioMes, $finMes]);
        })
        ->select(
            'users.id',
            'users.name',
            'users.email'
        )
        ->get()
        ->map(function($user){
            $evidencias = $user->participante->evidencias;
            $userDto = new UserActiveDto();
            $userDto->id = $user->id;
            $userDto->nombre = ($user->name) ? $user->name : $user->participante->nombre_completo;
            $userDto->email = $user->email;
            $userDto->isActive = (count($evidencias) > 0) ? true : false;
            return $userDto;
        });

        $users = $usersQuery->sortByDesc('isActive')->values();

        return $users;
    }

    public function findUser(string $idUser)
    {
        $user = User::findOrFail($idUser);
        return $user;
    }

    public function findUserByConfirmPassword(string $email, bool $confirm)
    {
        $userFind = User::where("email", $email)
            ->where("password_confirmed", $confirm)
            ->first();
        return $userFind;
    }

    public function findPerfilUser(string $userId)
    {
        $userQuery = User::leftJoin("participantes", "users.id", '=', "participantes.user_id")
            ->with("participante.files")
            ->where('users.id', $userId)
            ->select(
                "users.id",
                "users.name",
                "users.email",
                "participantes.nombre_completo",
                "participantes.localidad",
                "participantes.estado",
                "participantes.ciudad",
                "participantes.nombre_tienda",
                "participantes.direccion",
                "participantes.file_id"
            )
            ->first();
        return $userQuery;
    }

    public function deleteUser(int $idUser)
    {
        $userDelete = User::findOrfail($idUser);
        $userDelete->delete();
    }

    public function isUserWithRolAdmin(string $idUser)
    {
        $user = User::findOrFail($idUser);
        return $user->hasRole('administrador');
    }
}
