<?php

namespace App\Services;

use App\Dto\CanjeDto;
use App\Dto\Responses\ResponseCanjeDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseHistoryActionDto;
use App\Repositories\CanjeRepository;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class CanjeService
{
    protected $canjeRepository;

    public function __construct(CanjeRepository $canjeRepository)
    {
        $this->canjeRepository = $canjeRepository;
    }

    public function getCanjesAll()
    {
        try {
            $queryFilter = $this->canjeRepository->allRecompensasCanjeadas();
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar los canjes: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getCanjeDetail(string $idCanje)
    {
        try {
            $queryFilter = $this->canjeRepository->findCanje($idCanje);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al obtener el detalle del canje: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getCanjesByPlayer(string $idPlayer)
    {
        try {
            $queryFilter = $this->canjeRepository->findRecompensasCanjes($idPlayer);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al obtener los canjes del participante: ' . $e->getMessage(), ['exception' => $e, 'idPlayer' => $idPlayer ]);
            return new ResponseDataDto(false, null);
        }
    }


    public function getPointsCanjeByPlayer(string $idPlayer)
    {
        try {
            $queryFilter = $this->canjeRepository->pointsTotalCanjesByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al obtener los puntos del participante: ' . $e->getMessage(), ['exception' => $e, 'idPlayer' => $idPlayer ]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getPointsCanjeadosByRol(string $rol)
    {
        try {
            $queryFilter = $this->canjeRepository->pointsTotalCanjeadosByRol($rol);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al obtener los puntos canjeados globales: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function addCanje(CanjeDto $canjeDto)
    {
        try {
            $queryFilter = $this->canjeRepository->createCanje($canjeDto);
            $canjeDto->id = $queryFilter->id;
            return new ResponseCanjeDto(true, $canjeDto);
        } catch (\Exception $e) {
            Log::error('Error al crear canje: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseCanjeDto(false, null);
        }
    }

    public function updateStatusProcessCanje(CanjeDto $canjeDto, string $idCanje)
    {
        try {
            $itemCanje = $this->canjeRepository->updateCanje($canjeDto, $idCanje);
            return new ResponseCanjeDto(true, $itemCanje);
        } catch (\Exception $e) {
            Log::error('Error al actualizar el canje: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseCanjeDto(false, null);
        }
    }

    public function getHistoryActionCanjes(string $idPlayer)
    {
        try {
            $queryResult = $this->canjeRepository->historyActionCanjesRecompensa($idPlayer);
            return new ResponseHistoryActionDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar el historial de canjes obtenidos por el participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseHistoryActionDto(false, new Collection());
        }
    }
}
