<?php

namespace App\Services;

use App\Dto\HistorialRecompensaDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseHistorialRecompensaDto;
use App\Dto\Responses\ResponseHistoryActionDto;
use App\Repositories\HistorialRecompensaRepository;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class HistorialRecompensaService
{
    protected $historialRepository;

    public function __construct(HistorialRecompensaRepository $historialRepository)
    {
        $this->historialRepository = $historialRepository;
    }


    public function getHistorialRecompensasAll()
    {
        try {
            $queryResult = $this->historialRepository->allHistorialRecompensas();
            return new ResponseDataDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar los historiales de recompensas: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRecompensaDetail(string $idHistorial)
    {
        try {
            $queryResult = $this->historialRepository->findDetailRecompensa($idHistorial);
            return new ResponseDataDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar el detalle del historial recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getRecompensaHasInsigniaByPlayer(string $idPlayer)
    {
        try {
            $queryResult = $this->historialRepository->findRecompensasHasInsigniasByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar los historiales de recompensas por insignias del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getHistoryActionRecompensas(string $idPlayer)
    {
        try {
            $queryResult = $this->historialRepository->historyActionRecompensas($idPlayer);
            return new ResponseHistoryActionDto(true, $queryResult);
        } catch (\Exception $e) {
            Log::error('Error al consultar el historial de recompensas obtenidos por el participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseHistoryActionDto(false, new Collection());
        }
    }


    public function addHistorialRecompensa(HistorialRecompensaDto $historialRecompensaDto){
        try {
            $itemNew = $this->historialRepository->createHistorialRecompensa($historialRecompensaDto);
            $historialRecompensaDto->id = $itemNew->id;
            return new ResponseHistorialRecompensaDto(true, $historialRecompensaDto);
        } catch (\Exception $e) {
            Log::error('Error al crear el historial de recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseHistorialRecompensaDto(false, null);
        }
    }

    public function updateHistorialRecompensa(HistorialRecompensaDto $historialRecompensaDto, string $idHistorialRecompensa){
        try {
            $updateHistorialDto = $this->historialRepository->updateHistorialRecompensa($historialRecompensaDto, $idHistorialRecompensa);
            return new ResponseHistorialRecompensaDto(true, $updateHistorialDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar el historial recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseHistorialRecompensaDto(false, null);
        }
    }
}
