<?php

namespace App\Services;

use App\Dto\InsigniaDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseInsigniaDto;
use App\Repositories\InsigniaRepository;
use Illuminate\Support\Facades\Log;

class InsigniaService
{
    protected $insigniaRepository;

    public function __construct(InsigniaRepository $insigniaRepository)
    {
        $this->insigniaRepository = $insigniaRepository;
    }

    public function getInsigniasAll(){
        try {
            $allQuery = $this->insigniaRepository->allInsignia();
            return new ResponseDataDto(true, $allQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar las insignias: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getInsigniaByIdDetail(string $idInsignia){
        try {
            $itemQuery = $this->insigniaRepository->detailInsignia($idInsignia);
            return new ResponseDataDto(true, $itemQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar el detalle de la insignia: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getInsigniasByPlayer(string $idPlayer){
        try {
            $itemQuery = $this->insigniaRepository->findInsigniasByPlayer($idPlayer);
            return new ResponseDataDto(true, $itemQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar las insignias del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function addInsignia(InsigniaDto $insigniaDto){
        try {
            $itemNew = $this->insigniaRepository->createInsignia($insigniaDto);
            $insigniaDto->id = $itemNew->id;
            return new ResponseInsigniaDto(true, $insigniaDto);
        } catch (\Exception $e) {
            Log::error('Error al crear la insignia: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseInsigniaDto(false, null);
        }
    }

    public function updateInsignia(InsigniaDto $insigniaDto, int $idInsignia){
        try {
            $this->insigniaRepository->updateInsignia($insigniaDto, $idInsignia);
            return new ResponseInsigniaDto(true, $insigniaDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar la insignia: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseInsigniaDto(false, null);
        }
    }

    public function removeInsigniaById(string $idInsignia){
        try {
            $this->insigniaRepository->deleteInsignia($idInsignia);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar la insignia: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }
}
