<?php

namespace App\Services;

use App\Dto\PlayerDto;
use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponsePlayerDto;
use App\Repositories\PlayerRepository;
use Illuminate\Support\Facades\Log;

class PlayerService
{
    protected $playerRepository;

    public function __construct(PlayerRepository $playerRepository)
    {
        $this->playerRepository = $playerRepository;
    }

    public function getPlayersAll(){
        try {
            $playersAllQuery = $this->playerRepository->allPlayers();
            return new ResponseDataDto(true, $playersAllQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar los participantes: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getPlayersByRole(string $role){
        try {
            $playersQuery = $this->playerRepository->findPlayersByRole($role);
            return new ResponseDataDto(true, $playersQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar los participantes por rol: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getPlayerById(string $playerId){
        try {
            $playerQuery = $this->playerRepository->findPlayerById($playerId);
            return new ResponseDataDto(true, $playerQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar el detalle del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getPlayerByIdWithUserOnlyData(string $playerId){
        try {
            $playerQuery = $this->playerRepository->findPlayerWithUserOnly($playerId);
            return new ResponseDataDto(true, $playerQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar el participante con la información de su usuario: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getPlayerByUserId(string $userId){
        try {
            $playerQuery = $this->playerRepository->findPlayerByUserId($userId);
            return new ResponseDataDto(true, $playerQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar el participante por el id del usuario: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function getPointsRankingByRolPlayer(string $typePlayer){
        try {
            $playerQuery = $this->playerRepository->getRankingByTypePlayer($typePlayer);
            return new ResponseDataDto(true, $playerQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar el ranking por rol de participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function addPlayer(PlayerDto $playerDto){
        try {
            $participanteNew = $this->playerRepository->createPlayer($playerDto);
            $playerDto->setId($participanteNew->id);
            return new ResponsePlayerDto(true, $playerDto);
        } catch (\Exception $e) {
            Log::error('Error al crear participante en el register: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponsePlayerDto(false, null);
        }
    }

    public function updatePlayer(PlayerDto $playerDto, int $idPlayer){
        try {
            $participanteUpdate = $this->playerRepository->updatePlayer($playerDto, $idPlayer);
            $playerDto->setId($participanteUpdate->id);
            return new ResponsePlayerDto(true, $playerDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponsePlayerDto(false, null);
        }
    }

    public function updatePlayerByUserId(PlayerDto $playerDto, int $idUser){
        try {
            $participanteUpdate = $this->playerRepository->updatePlayerFindUserId($playerDto, $idUser);
            $playerDto->setId($participanteUpdate->id);
            return new ResponsePlayerDto(true, $playerDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar participante por el usuario: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponsePlayerDto(false, null);
        }
    }

    public function removePlayerById(string $idPlayer){
        try {
            $this->playerRepository->deletePlayer($idPlayer);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar participante: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }
}
