<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\V1\Seguridad\AuthController;
use App\Http\Controllers\V1\Seguridad\ForgotPasswordController;
use App\Http\Controllers\V1\Seguridad\ResetPasswordController;
use App\Http\Controllers\V1\Producto\ProductoController;
use App\Http\Controllers\V1\Participante\ParticipanteController;
use App\Http\Controllers\V1\Insignia\InsigniaController;
use App\Http\Controllers\V1\File\FileController;
use App\Http\Controllers\V1\Reto\RetoController;
use App\Http\Controllers\V1\Evidencia\EvidenciaController;
use App\Http\Controllers\V1\Repartidor\RepartidorController;
use App\Http\Controllers\V1\Recompensa\RecompensaController;
use App\Http\Controllers\V1\Tendero\TenderoController;
use App\Http\Controllers\V1\Reto\RetoParticipanteController;
use App\Http\Controllers\V1\Perfil\PerfilController;
use App\Http\Controllers\V1\Historial\HistorialRecompensaController;
use App\Http\Controllers\V1\Canje\CanjeController;



/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/


Route::prefix('auth')->group(function () {
    Route::post('logout', [AuthController::class, 'logout']);
    Route::post('login', [AuthController::class, 'authenticate']);
    Route::post('get-user', [AuthController::class, 'getUser']);
    Route::post('register', [AuthController::class, 'register'])->middleware('role_or_permission:administrador');
    Route::put('confirmPassword', [AuthController::class, 'confirmarUsuario']);
    Route::put('updateCountSession/{userId}', [AuthController::class, 'updateIncrementCountSession']);
    Route::get('get-users-history-month-active', [AuthController::class, 'getDataUserActive']);
});

Route::post('password/email', [ForgotPasswordController::class, 'sendResetLinkEmail']);
Route::post('password/reset', [ResetPasswordController::class, 'reset'])->name('password.reset');

Route::prefix('participantes')->middleware('auth:api')->group(function () {
    //obtiene los participantes
    Route::get("all", [ParticipanteController::class, 'index'])->middleware('role_or_permission:administrador');
    //obtiene la informacion de un participante
    Route::get("detail/{id}", [ParticipanteController::class, 'show'])->middleware('role_or_permission:administrador');
    //crea un participante
    Route::post("create", [ParticipanteController::class, 'store'])->middleware('role_or_permission:administrador');
    //actualiza participante
    Route::put("update/{id}", [ParticipanteController::class, 'update'])->middleware('role_or_permission:administrador');
    //elimina un participante
    Route::delete("delete/{id}", [ParticipanteController::class, 'destroy'])->middleware('role_or_permission:administrador');
    //obtiene los participantes por role
    Route::get("allByRole/{role}", [ParticipanteController::class, 'getParticipanteRole'])->middleware('role_or_permission:administrador|repartidor');
});

Route::prefix('productos')->middleware('auth:api')->group(function () {
    //obtiene los productos
    Route::get("all", [ProductoController::class, 'index'])->middleware('role_or_permission:administrador|repartidor');
    //obtiene la informacion de un producto
    Route::get("detail/{id}", [ProductoController::class, 'show'])->middleware('role_or_permission:administrador');
    //crea un producto
    Route::post("create", [ProductoController::class, 'store'])->middleware('role_or_permission:administrador');
    //actualiza producto
    Route::put("update/{id}", [ProductoController::class, 'update'])->middleware('role_or_permission:administrador');
    //elimina un producto
    Route::delete("delete/{id}", [ProductoController::class, 'destroy'])->middleware('role_or_permission:administrador');
});

Route::prefix('insignias')->middleware('auth:api')->group(function () {
    //obtiene las insignias
    Route::get("all", [InsigniaController::class, 'index'])->middleware('role_or_permission:administrador|tendero|repartidor');
    //obtiene la informacion de una insignia
    Route::get("detail/{id}", [InsigniaController::class, 'show'])->middleware('role_or_permission:administrador');
    //crea una insignia
    Route::post("create", [InsigniaController::class, 'store'])->middleware('role_or_permission:administrador');
    //actualiza insignia
    Route::put("update/{id}", [InsigniaController::class, 'update'])->middleware('role_or_permission:administrador');
    //elimina una insignia
    Route::delete("delete/{id}", [InsigniaController::class, 'destroy'])->middleware('role_or_permission:administrador');
});

//apis files
Route::prefix('files')->middleware('auth:api')->group(function () {
    //obtiene archivos
    Route::get('all/{idUser}', [FileController::class, 'index'])->middleware('role_or_permission:administrador|repartidor|tendero');
    //sube un FileController
    Route::post('create', [FileController::class, 'store'])->middleware('role_or_permission:administrador|repartidor|tendero');
    //elimina un archivo
    Route::delete('delete/{idArchivo}', [FileController::class, 'destroy'])->middleware('role_or_permission:administrador|repartidor|tendero');
});

Route::prefix('retos')->middleware('auth:api')->group(function () {
    //obtiene los retos
    Route::get("all", [RetoController::class, 'index'])->middleware('role_or_permission:administrador');
    //obtiene los retos por tipo de participante ya sea tendero o repartidor
    Route::get("retosByTypeParticipante/{tipo_reto}", [RetoController::class, 'retosByTypeParticipante'])->middleware('role_or_permission:administrador');
    //obtiene la informacion de un producto
    Route::get("detail/{id}", [RetoController::class, 'show'])->middleware('role_or_permission:administrador');
    //crea un producto
    Route::post("create", [RetoController::class, 'store'])->middleware('role_or_permission:administrador');
    //actualiza producto
    Route::put("update/{id}", [RetoController::class, 'update'])->middleware('role_or_permission:administrador');
    //elimina un producto
    Route::delete("delete/{id}", [RetoController::class, 'destroy'])->middleware('role_or_permission:administrador');
    //asignar retos
    Route::post("asignReto", [RetoController::class, 'asignReto'])->middleware('role_or_permission:administrador');
    //consultar avance de retos por rol
    Route::get("get-advance-retos-rol/{rol}", [RetoController::class, 'getAvanceRetosParticipantes'])->middleware('role_or_permission:administrador');
});

Route::prefix('evidencias')->middleware('auth:api')->group(function () {
    //obtiene las evidencias
    Route::get("all", [EvidenciaController::class, 'index'])->middleware('role_or_permission:repartidor|administrador');
    //obtiene la informacion de una evidencia
    Route::get("detail/{id}", [EvidenciaController::class, 'show'])->middleware('role_or_permission:repartidor|administrador|tendero');
    //crea una evidencia
    Route::post("create", [EvidenciaController::class, 'store'])->middleware('role_or_permission:repartidor|administrador');
    //crea evidencias multiples
    Route::post("createMultiple", [EvidenciaController::class, 'storeMultiple'])->middleware('role_or_permission:repartidor|administrador');
    //actualiza una evidencia por su id
    Route::put("update/{id}", [EvidenciaController::class, 'update'])->middleware('role_or_permission:repartidor');
});

Route::prefix('repartidor')->middleware('auth:api')->group(function () {
    //obtiene las evidencias del usuario y reto
    // Route::get("getEvidencias/{idUser}", [RepartidorController::class, 'getEvidencias'])->middleware('role_or_permission:repartidor');
    Route::post("createRecompensa", [HistorialRecompensaController::class, 'store'])->middleware('role_or_permission:repartidor');
});


Route::prefix('recompensas')->middleware('auth:api')->group(function () {
    Route::get("get-recompensas-active", [RecompensaController::class, 'getRecompensasActivas'])->middleware('role_or_permission:administrador|tendero|repartidor');
    //obtiene las recompensas
    Route::get("all", [RecompensaController::class, 'index'])->middleware('role_or_permission:administrador|tendero|repartidor');
    //obtiene la informacion de una recompensa
    Route::get("detail/{id}", [RecompensaController::class, 'show'])->middleware('role_or_permission:administrador|tendero|repartidor');
    //crea una recompensa
    Route::post("create", [RecompensaController::class, 'store'])->middleware('role_or_permission:administrador');
    //actualiza recompensa
    Route::put("update/{id}", [RecompensaController::class, 'update'])->middleware('role_or_permission:administrador');
    //elimina una recompensa
    Route::delete("delete/{id}", [RecompensaController::class, 'destroy'])->middleware('role_or_permission:administrador');
});


Route::prefix('historial-recompensas')->middleware('auth:api')->group(function () {
    //consulta premios
    Route::get("all", [HistorialRecompensaController::class, 'index'])->middleware('role_or_permission:administrador');
    //obtiene la informacion de una recompensa
    Route::get("detail/{id}", [HistorialRecompensaController::class, 'show'])->middleware('role_or_permission:repartidor|administrador|tendero');
    Route::put("updateRecompensa/{id}", [HistorialRecompensaController::class, 'update'])->middleware('role_or_permission:administrador');
});

Route::prefix('tendero')->middleware('auth:api')->group(function () {
});

Route::prefix('canjes')->middleware('auth:api')->group(function () {
    //consulta canjes recompensas
    Route::get("get-canjes/", [CanjeController::class, 'getRecompensasCanje'])->middleware('role_or_permission:administrador');
    //consulta canjes by participante
    Route::get("get-canjes-participante/{idParticipante}", [CanjeController::class, 'getRecompensasCanjeByParticipante'])->middleware('role_or_permission:tendero|repartidor');
    //consulta detalle del canje
    Route::get("detail-canje/{idCanje}", [CanjeController::class, 'getCanjeDetalle'])->middleware('role_or_permission:administrador|tendero|repartidor');
    //consulta puntos de canje recompensas
    Route::get("get-points-player/{idParticipante}", [CanjeController::class, 'getPuntosCanjeParticipante'])->middleware('role_or_permission:tendero|repartidor');
    //crea una caje
    Route::post("create", [CanjeController::class, 'createCanjeParticipante'])->middleware('role_or_permission:tendero|repartidor');
    //actualizar canje estatus
    Route::put("update-status", [CanjeController::class, 'changeStatusCanje'])->middleware('role_or_permission:administrador');
});

Route::prefix('perfil')->middleware('auth:api')->group(function () {
    Route::get("getPerfil/{userId}", [PerfilController::class, 'getPerfil'])->middleware('role_or_permission:repartidor|administrador|tendero');
    Route::put("updatePerfil/", [PerfilController::class, 'updatePerfil'])->middleware('role_or_permission:repartidor|administrador|tendero');
});

Route::prefix('participante_reto')->middleware('auth:api')->group(function () {
    //obtiene las evidencias del participante por reto, mas sus puntos totales
    Route::get("getEvidencias/{idParticipante}", [RetoParticipanteController::class, 'getEvidencias'])->middleware('role_or_permission:tendero|repartidor');
    //obtiene retos vigentes
    Route::get("getRetosVigentes/{idParticipante}", [RetoParticipanteController::class, 'retosVigentes'])->middleware('role_or_permission:tendero|repartidor');
    //consulta premios
    Route::get("getRecompensas/{idParticipante}", [RetoParticipanteController::class, 'getRecompensas'])->middleware('role_or_permission:tendero|repartidor');
    //consulta cantidad total de venta y monto por participante
    Route::get("getTotalVentaProductosByParticipante/{idParticipante}", [RetoParticipanteController::class, 'getTotalVentaProductosByParticipante'])->middleware('role_or_permission:tendero|repartidor');
    Route::get("getTotalVentaProductos/", [RetoParticipanteController::class, 'getTotalVentaProductosGlobal'])->middleware('role_or_permission:administrador');
    //consulta cantidad total puntos del participante
    Route::get("getTotalPuntosByParticipante/{idParticipante}", [RetoParticipanteController::class, 'getTotalPuntosByParticipante'])->middleware('role_or_permission:tendero|repartidor');
    //consulta cantidad total de puntos globales por rol
    Route::get("getTotalPuntosGlobalesByRol/{role}", [RetoParticipanteController::class, 'getTotalPuntosGlobalesByRole'])->middleware('role_or_permission:administrador|tendero|repartidor');
    //tabla de puntuacion por monto venta, puntos, cantidad de venta
    Route::get("getRankingByTypePlayers/{tipo_participante}", [RetoParticipanteController::class, 'getRankingByTypePlayers'])->middleware('role_or_permission:tendero|repartidor|administrador');
    //consulta insigniass
    Route::get("getInsigniasByParticipante/{idParticipante}", [RetoParticipanteController::class, 'getInsigniasByParticipante'])->middleware('role_or_permission:tendero|repartidor');
    //consulta retos asignados del participante
    Route::get("getRetosByParticipante/{idParticipante}", [RetoParticipanteController::class, 'getRetosAsignadosParticipante'])->middleware('role_or_permission:tendero|repartidor');
    //consulta retos detalle de un participante
    Route::get("getRetoDetalleParticipante/{idParticipante}/{idReto}", [RetoParticipanteController::class, 'getRetoDetalleParticipante'])->middleware('role_or_permission:tendero|repartidor');
    Route::get("getHistoryActionRecompensas/{idParticipante}", [RetoParticipanteController::class, 'getHistoryReportActionByPlayer'])->middleware('role_or_permission:administrador|tendero|repartidor');
    
});

Route::prefix('manage_evidencias')->middleware('auth:api')->group(function () {
    //actualiza el estatus de la evidencia
    Route::put("evaluate/{id}", [EvidenciaController::class, 'evaluarEvidencia'])->middleware('role_or_permission:administrador');
});
